use 5.006;    # our
use strict;
use warnings;

package       ## no index
  Example::Inc::in::INC;

our $VERSION = '0.001000';

# ABSTRACT: An example package replicating the common "dist tools in inc/" setup

# AUTHORITY

1;

